'use strict';

var RWADODataAuth = angular.module('RWADODataAuth_plugin', ['ngCookies', 'sharedProvidersModule']);

RWADODataAuth.factory('authenticationSvc', function (Base64, $http, $cookieStore, $rootScope, $timeout, settingsSvc) {
    var service = {};

    service.authConfig = settingsSvc.GetAuthConfig();

    service.Login = function (username, password, callback) {

        var loginData = {
            Login: username,
            Password: password
        };

        var authdata =  Base64.encode(username + ':' + password);

        var basicLogin = function() {
            var requestConfig = {
                url: service.authConfig.url,
                method: 'POST',
                headers: {
                    'Authorization': 'Basic ' + authdata,
                },                
                data: loginData,
                withCredentials: true
            };

            $http(requestConfig)
            .then(function (response) {
                callback(response);
            }, function(err) {
                return callback(err);
            });
        }

        var tokenLogin = function() {
            var requestConfig = {
                url: service.authConfig.url,
                method: 'POST',
                data: {
                    Login: username,
                    Password: password
                }
            };

            $http(requestConfig)
            .then(function (response) {
                if (service.authConfig.tokenStoreMethod == 'Header')
                    $http.defaults.headers.common['Authorization'] = response.data;
                callback(response);
            }, function(err) {
                return callback(err);
            });
        }

        
        if (service.authConfig.authType === 'Basic')
            basicLogin();    
        else
            tokenLogin();
    };

    service.SetCredentials = function (username, password) {
        var authdata = Base64.encode(username + ':' + password);
        if (service.authConfig.authType === 'Token')
            authdata = $http.defaults.headers.common['Authorization'];
        else
            $http.defaults.headers.common['Authorization'] = 'Basic ' + authdata;
            
        $rootScope.globals = {
            currentUser: {
                username: username,
                authdata: authdata
            }
        };
        
        $cookieStore.put(document.ProjectName, $rootScope.globals);
    };

    service.ClearCredentials = function() {
        $rootScope.globals = {};
        
        $cookieStore.remove(document.ProjectName);

        if ((service.authConfig.authType === 'Basic') || (service.authConfig.authType === 'Token' && service.authConfig.tokenStoreMethod == 'Header'))
            $http.defaults.headers.common.Authorization = '';
    };

    return service;
});


RWADODataAuth.factory('authServiceBase', ['$http', 'settingsSvc',
function ($http, settingsSvc) {
    function performGetOperation(targetUrl, passedParam) {
        return $http.get(targetUrl).then(
        function(response) {
            if (response.status === 401 || response.status === 403) 
                return  {
                    error: true,
                    message: response.data
                }
            else
                return response.data;
        },
        function(error) {
            return error.data;
        });
    };

    function performPostOperation(targetUrl, passedParam) {
        return $http.post(targetUrl, passedParam).then(
        function(response) {
            if (response.status === 401 || response.status === 403) 
                return  {
                    error: true,
                    message: response.data
                }
            else
                return response.data;
        },
        function(error) {
            return error.data;
        });
    };

    return {
        PerformGetOperation: performGetOperation,
        PerformPostOperation: performPostOperation
    };
}]);

RWADODataAuth.factory('usersService', ['$rootScope', '$http', 'authServiceBase', 'settingsSvc',
function ($rootScope, $http, authServiceBase, settingsSvc) {
    var service = Object.create(authServiceBase);

    service.GetUsersList = function() {
        var targetUrl = '{0}GetAllUsers'.format(settingsSvc.GetWSRootHost());
        return  service.PerformGetOperation(targetUrl);
    };

    service.CreateUser = function(User) {
        var targetUrl = '{0}CreateUser'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, User);
    };    

    service.UpdateUser = function(User) {
        var targetUrl = '{0}UpdateUser'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, User);
    };

    service.DeleteUser = function(userID) {
        var targetUrl = '{0}DeleteUser'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, { UserID: userID });
    };

    service.UpdateUserPassword = function(userLogin, newPass) {
        var targetUrl = '{0}UpdateUserPassword'.format(settingsSvc.GetWSRootHost());

        // Если меняем пароль для текущего пользака - то запрашиваем другой маршрут (не требующий права редактирования сущности User)
        if (userLogin === $rootScope.globals.currentUser.username)
            targetUrl = '{0}UpdateMyPassword'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, { UserLogin: userLogin, NewPassword: newPass });
    };

    return service;
}]);

RWADODataAuth.factory('rolesService', ['$http', 'authServiceBase', 'settingsSvc',
function ($http, authServiceBase, settingsSvc) {
    var service = Object.create(authServiceBase);

    service.GetRolesList = function() {
        var targetUrl = '{0}GetAllRoles'.format(settingsSvc.GetWSRootHost());
        return  service.PerformGetOperation(targetUrl);
    };

    service.CreateRole = function(Role) {
        var targetUrl = '{0}CreateRole'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, Role);
    };    

    service.UpdateRole = function(Role) {
        var targetUrl = '{0}UpdateRole'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, Role);
    };

    service.DeleteRole = function(roleID) {
        var targetUrl = '{0}DeleteRole'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, { RoleID: roleID });
    };

    service.GetRolePermissions = function(roleID) {
        var targetUrl = '{0}GetRolePermissions'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, { RoleID: roleID });
    };

    service.ApplyRolePermissions = function(roleID, enabledPermissionsIdList) {
        var targetUrl = '{0}ApplyRolePermissions'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, { RoleID: roleID, EnabledPermissionsIDsList: enabledPermissionsIdList });
    };

    return service;
}]);

RWADODataAuth.factory('membershipService', ['$http', 'authServiceBase', 'settingsSvc', 'rolesService',
function ($http, authServiceBase, settingsSvc, rolesService) {
    var service = Object.create(authServiceBase);

    service.GetRolesIDsByUser = function(userID) {
        var targetUrl = '{0}GetRolesByUser'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, { UserID: userID });
    };

    service.ApplyUserRoles = function(userID, roleIDsList) {
        var targetUrl = '{0}ApplyUserRoles'.format(settingsSvc.GetWSRootHost());
        return  service.PerformPostOperation(targetUrl, { UserID: userID, RoleIDsList: roleIDsList });
    };

    return service;
}]);